"
===================================================
Copyright  1999 Moshe Brody, All Rights Reserved.
===================================================
"
PROJECTNAME "Split Pane Limits" .
PROJECTCATEGORY Extensions .


PROFILE
BEGIN
END

CLASS SplitPane
INCLUDEDEF=0
CLASSMETHODS 
BEGIN
END
INSTANCEMETHODS 
BEGIN
	checkSplitPosition
	minInfo
	splitHAt:
	splitVAt:
END

CLASS FrameWindow
INCLUDEDEF=0
CLASSMETHODS 
BEGIN
END
INSTANCEMETHODS 
BEGIN
	resize
END
! !

!FrameWindow Update methods 01:32 - 05/30/99!
resize

"
Forces the receiver window to resize the child windows.
Return Value:
	This method does not return a value.

23.05.99 MDB --	update split panes, if any, upon a resize.
"

| rcAvailable split |

(split := self childAt: ID_SPLITPANE) notNil
	ifTrue: [split checkSplitPosition].
rcAvailable := WinRectangle localNew.
WINAPI GetClientRect: m_handle with: rcAvailable basicAddress.
self preSize: rcAvailable update: true.
^self reframe_x: rcAvailable left y: rcAvailable top cx: rcAvailable width cy: rcAvailable height.! !

!SplitPane Updating methods 01:32 - 05/30/99!
checkSplitPosition

"For a resize operation on the parent window, check to see if the receiver needs its split position updated to
assure minimum split requirements. If so, adjust its split position accordingly."

| rc position extent x y |

((m_minInfo top < 0) | (m_minInfo bottom < 0) | (m_minInfo left < 0) | (m_minInfo right < 0)) ifFalse: [^nil].		"no minimum split"

position := self position.
rc := WinRectangle localNew.
extent := (m_wndParent getAvailableClientRect: rc) extent.

x := position x.
((x < m_minInfo left negated) | (x > (extent x + m_minInfo right)))
	ifTrue: [self splitVAt: x].
y := position y.
((y < m_minInfo top negated) | (y > (extent y + m_minInfo bottom)))
	ifTrue: [self splitHAt: y].! !

!SplitPane Accessing methods 10:40 - 05/23/99!
minInfo

"
Answers the minimum extent.
Return Value:
	A Rectangle.
Remarks:


The receiver hides panes that are resized below the minimum extent values. Initially, the
minimum extent is based on the system scrollbar width and height. An application
can modify the left, right, top, and bottom values of m_minInfo.

23.05.99 MDB --	Corrected comment ('Point' should be 'Rectangle').
		Extension: If m_minInfo left < 0, it means that the split cannot be moved left beyond this.
		Likewise for right, top, and bottom. For each extreme, this enables you to specify either
		1.	a position below which the pane is hidden; or
		2.	a position below which the split cannot be moved
"

^m_minInfo! !

!SplitPane Private methods 01:32 - 05/30/99!
splitHAt: y

"23.05.99 MDB --	Extend to handle negative values of m_minInfo parameters."
	
| fm fp extent ids rc fmScale newYT newYB |

rc := WinRectangle localNew.

extent := (m_wndParent getAvailableClientRect: rc) extent.
fp := m_wndParent framingParameters.

m_hideH ~~ 0
	ifTrue:
	[
		ids := m_panes _at: (m_hideH == SPLIT_1 ifTrue: [1] ifFalse: [2]).
		1 to: ids size do:
			[
				:i |
				WINAPI ShowWindow: (WINAPI GetDlgItem: m_wndParent with: (ids _at: i)) with: SW_SHOW.
			].
		" restore framing matrices "
		ids := m_panes _at: (m_hideH == SPLIT_1 ifTrue: [2] ifFalse: [1]).
		1 to: m_hMax size do:
			[
				:i |
				fm := fp at: (ids _at: i).
				fmScale := m_hMax _at: i.
				fm _at: ##(Compiler varIndex: 's2' asStringA in: 'FrameMatrix' asStringA) put: (fmScale _at: ##(Compiler varIndex: 's2' asStringA in: 'FrameMatrix' asStringA)).
				fm _at: ##(Compiler varIndex: 'o2' asStringA in: 'FrameMatrix' asStringA) put: (fmScale _at: ##(Compiler varIndex: 'o2' asStringA in: 'FrameMatrix' asStringA)).
				fm _at: ##(Compiler varIndex: 's4' asStringA in: 'FrameMatrix' asStringA) put: (fmScale _at: ##(Compiler varIndex: 's4' asStringA in: 'FrameMatrix' asStringA)).
			].
		m_hMax := nil.
		m_hideH := 0.
	].


(m_minInfo top > 0)
	ifTrue:
	[
		y < m_minInfo top
			ifTrue:
			[
				" hide the upper windows and maximize bottom windows to y "
				m_hideH := SPLIT_1.
				^m_hMax := self hidePanes: (m_panes _at: 1) maximizeH: (m_panes _at: 2).
			].
	].

(m_minInfo bottom > 0)
	ifTrue:
	[
		(extent y - y) < m_minInfo bottom
			ifTrue:
			[
				" insert split at top, scale top windows to frame, and hide bottom windows "
				m_hideH := SPLIT_2.
				^m_hMax := self hidePanes: (m_panes _at: 2) maximizeH: (m_panes _at: 1).
			].
	].

newYT := newYB := y.
(m_minInfo top < 0)
	ifTrue:
	[
		(y <  m_minInfo top negated)
			ifTrue:
			[
				newYT := m_minInfo top negated.
				newYB := newYT - 1.
			]
	].

(m_minInfo bottom < 0)
	ifTrue:
	[
		(y > (extent y + m_minInfo top))
			ifTrue:
			[
				newYT := extent y + m_minInfo top.
				newYB := newYT - 1.
			]
	].

ids := m_panes _at: 1.
1 to: ids size do:
	[
		:i |
		fm := fp at: (ids _at: i).
		fm changeBottomTo: newYB in: extent
	].

ids := m_panes _at: 2.
1 to: ids size do:
	[
		:i |
		fm := fp at: (ids _at: i).
		fm changeTopTo: newYT in: extent
	].! !

!SplitPane Private methods 01:32 - 05/30/99!
splitVAt: x

"23.05.99 MDB --	Extend to handle negative values of m_minInfo parameters."

| fm fp extent ids rc fmScale newXL newXR |

rc := WinRectangle localNew.

extent := (m_wndParent getAvailableClientRect: rc) extent.
fp := m_wndParent framingParameters.

m_hideV ~~ 0
	ifTrue:
	[
		" restore split "
		ids := m_panes _at: (m_hideV == SPLIT_1 ifTrue: [3] ifFalse: [4]).
		1 to: ids size do:
			[
				:i |
				WINAPI ShowWindow: (WINAPI GetDlgItem: m_wndParent with: (ids _at: i)) with: SW_SHOW.
			].
		" restore framing matrices "
		ids := m_panes _at: (m_hideV == SPLIT_1 ifTrue: [4] ifFalse: [3]).
		1 to: m_vMax size do:
			[
				:i |
				fm := fp at: (ids _at: i).
				fmScale := m_vMax _at: i.

				fm _at: ##(Compiler varIndex: 's1' asStringA in: 'FrameMatrix' asStringA) put: (fmScale _at: ##(Compiler varIndex: 's1' asStringA in: 'FrameMatrix' asStringA)).
				fm _at: ##(Compiler varIndex: 'o1' asStringA in: 'FrameMatrix' asStringA) put: (fmScale _at: ##(Compiler varIndex: 'o1' asStringA in: 'FrameMatrix' asStringA)).
				fm _at: ##(Compiler varIndex: 's3' asStringA in: 'FrameMatrix' asStringA) put: (fmScale _at: ##(Compiler varIndex: 's3' asStringA in: 'FrameMatrix' asStringA)).
			].
		m_vMax := nil.
		m_hideV := 0.
	].

(m_minInfo left > 0)
	ifTrue:
	[
		x < m_minInfo left
			ifTrue:
			[
				" hide the left windows and maximize right windows to x "
				m_hideV := SPLIT_1.
				^m_vMax := self hidePanes: (m_panes _at: 3) maximizeV: (m_panes _at: 4).
			]
	].

(m_minInfo right > 0)
	ifTrue:
	[
		(extent x - x) < m_minInfo right
			ifTrue:
			[ 
				" insert split at left, scale left windows to frame, and hide right windows "
				m_hideV := SPLIT_2.
				^m_vMax := self hidePanes: (m_panes _at: 4) maximizeV: (m_panes _at: 3).
			]
	].

newXL := newXR := x.
(m_minInfo left < 0)
	ifTrue:
	[
		(x < m_minInfo left negated)
			ifTrue:
			[
				newXL := m_minInfo left negated.
				newXR := newXL - 1.
			]
	].


(m_minInfo right < 0)
	ifTrue:
	[
		(x > (extent x + m_minInfo right))
			ifTrue:
			[
				newXL := extent x + m_minInfo right.
				newXR := newXL - 1.
			].
	].

ids := m_panes _at: 3.
1 to: ids size do:
	[
		:i |
		fm := fp at: (ids _at: i).
		fm changeRightTo: newXR in: extent
	].

ids := m_panes _at: 4.
1 to: ids size do:
	[
		:i |
		fm := fp at: (ids _at: i).
		fm changeLeftTo: newXL in: extent
	].! !

Compiler addCatDescription: 'Accessing' text: 'Accessing
Accesses properties of the object.'.
Compiler addCatDescription: 'Update' text: 'Update'.
Compiler addCatDescription: 'Private' text: 'Private
Private methods.'.
Compiler addCatDescription: 'Updating' text: 'Updating'.!
